@@ -31,8 +31,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/statistics.h>
 
 #include "main.h"
-
-
+#include "convertt.h"
 
 
 
@@ -102,13 +101,13 @@ color_from_mono_hsv(struct converttparams *p)
   /* Allocate the three datasets to keep the RGB colors. */
   R=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, channel->ndim,
                    channel->dsize, channel->wcs, 0, p->cp.minmapsize,
-                   "RED", NULL, "Red color channel.");
+                   p->cp.quietmmap, "RED", NULL, "Red color channel.");
   G=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, channel->ndim,
                    channel->dsize, channel->wcs, 0, p->cp.minmapsize,
-                   "GREEN", NULL, "Green color channel.");
+                   p->cp.quietmmap, "GREEN", NULL, "Green color channel.");
   B=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, channel->ndim,
                    channel->dsize, channel->wcs, 0, p->cp.minmapsize,
-                   "BLUE", NULL, "Blue color channel.");
+                   p->cp.quietmmap, "BLUE", NULL, "Blue color channel.");
 
   /* Start the conversion. Note that the "Choroma" (`C') is fixed by our
      definition. */
@@ -188,13 +187,13 @@ color_from_mono_sls(struct converttparams *p)
   /* Allocate the three datasets to keep the RGB colors. */
   R=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, channel->ndim,
                    channel->dsize, channel->wcs, 0, p->cp.minmapsize,
-                   "RED", NULL, "Red color channel.");
+                   p->cp.quietmmap, "RED", NULL, "Red color channel.");
   G=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, channel->ndim,
                    channel->dsize, channel->wcs, 0, p->cp.minmapsize,
-                   "GREEN", NULL, "Green color channel.");
+                   p->cp.quietmmap, "GREEN", NULL, "Green color channel.");
   B=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, channel->ndim,
                    channel->dsize, channel->wcs, 0, p->cp.minmapsize,
-                   "BLUE", NULL, "Blue color channel.");
+                   p->cp.quietmmap, "BLUE", NULL, "Blue color channel.");
 
   /* Start the conversion. Note that the "Choroma" (`C') is fixed by our
      definition. */
@@ -435,6 +434,25 @@ color_from_mono_sls(struct converttparams *p)
 
 
 
+void
+color_map_prepare(struct converttparams *p)
+{
+  switch(p->colormap->status)
+    {
+    case COLOR_HSV:  color_from_mono_hsv(p);     break;
+    case COLOR_SLS:  color_from_mono_sls(p);     break;
+    case COLOR_GRAY: convertt_scale_to_uchar(p); break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
+            "the problem. The value %d is not a recognized color-space "
+            "code", __func__, PACKAGE_BUGREPORT, p->colormap->status);
+    }
+}
+
+
+
+
+
 
 
 
@@ -475,13 +493,13 @@ color_rgb_to_hsv(struct converttparams *p)
   /* Allocate the three datasets to keep the RGB colors. */
   H=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, p->chll->ndim,
                    p->chll->dsize, p->chll->wcs, 0, p->cp.minmapsize,
-                   "HUE", NULL, NULL);
+                   p->cp.quietmmap, "HUE", NULL, NULL);
   S=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, p->chll->ndim,
                    p->chll->dsize, p->chll->wcs, 0, p->cp.minmapsize,
-                   "SATURATION", NULL, NULL);
+                   p->cp.quietmmap, "SATURATION", NULL, NULL);
   V=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, p->chll->ndim,
                    p->chll->dsize, p->chll->wcs, 0, p->cp.minmapsize,
-                   "VALUE", NULL, NULL);
+                   p->cp.quietmmap, "VALUE", NULL, NULL);
 
   /* Initiate the pointer arrays. */
   h=H->array;
