@@ -3,9 +3,9 @@ Crop - Crop a given size from one or multiple images.
 Crop is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017, Free Software Foundation, Inc.
+Copyright (C) 2017-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -31,8 +31,8 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
    See the `gnuastro_bibtex' variable in `lib/options' (from the top
    Gnuastro source code directory) as an example.*/
 
-#define PROGRAM_BIBTEX "";
+#define PROGRAM_BIBTEX ""
 
-#define PROGRAM_AUTHORS "Mohammad Akhlaghi";
+#define PROGRAM_AUTHORS "Mohammad Akhlaghi"
 
 #endif
