@@ -5,7 +5,7 @@ Crop is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2018, Free Software Foundation, Inc.
+Copyright (C) 2016-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -66,7 +66,7 @@ const char *
 argp_program_bug_address = PACKAGE_BUGREPORT;
 
 static char
-args_doc[] = "[Crop-identifiers] ASTRdata ...";
+args_doc[] = "[Crop-Identifier] ASTRdata ...";
 
 const char
 doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" will create cutouts, "
@@ -359,9 +359,9 @@ ui_read_check_only_options(struct cropparams *p)
          given, so we only need to check one of the two in each couple. */
       if(p->coordcol==NULL)
         error(EXIT_FAILURE, 0, "no crop center columns given to read from "
-              "the input catalog (`%s'). Please use `--coordcol' two times "
-              "to specify the column keeping the center position the "
-              "respective dimension.\n\n"
+              "the input catalog (`%s'). Please use `--coordcol' several "
+              "times (depending on dimensionality) to specify the column "
+              "keeping the center position the respective dimension.\n\n"
               "For more information on how to select columns in Gnuastro, "
               "please run the following command:\n\n"
               "    $ info gnuastro \"Selecting table columns\"", p->catname);
@@ -516,7 +516,7 @@ ui_set_img_sizes(struct cropparams *p)
     {
       /* Allocate the new width dataset. */
       newwidth=gal_data_alloc(NULL, p->width->type, 1, &ndim, NULL, 0, -1,
-                              NULL, NULL, NULL);
+                              1, NULL, NULL, NULL);
 
       /* Fill the new width. */
       warray=newwidth->array;
@@ -676,7 +676,8 @@ ui_read_cols(struct cropparams *p)
 
   /* Read the desired columns from the file. */
   cols=gal_table_read(p->catname, p->cathdu, NULL, colstrs, p->cp.searchin,
-                      p->cp.ignorecase, p->cp.minmapsize, NULL);
+                      p->cp.ignorecase, p->cp.minmapsize, p->cp.quietmmap,
+                      NULL);
   if(cols==NULL)
     error(EXIT_FAILURE, 0, "%s: is empty! No usable information "
           "(un-commented lines) could be read from this file",
@@ -796,7 +797,6 @@ ui_prepare_center(struct cropparams *p)
 
 
 
-
 /* Add all the columns of the log file. Just note that since this is a
    linked list, we have to add them in the opposite order. */
 static void
@@ -812,19 +812,20 @@ ui_make_log(struct cropparams *p)
                "%u: not checked)", GAL_BLANK_UINT8)<0 )
     error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   gal_list_data_add_alloc(&p->log, NULL, GAL_TYPE_UINT8, 1, &p->numout,
-                          NULL, 1, p->cp.minmapsize, "CENTER_FILLED",
-                          "bool", comment);
+                          NULL, 1, p->cp.minmapsize, p->cp.quietmmap,
+                          "CENTER_FILLED", "bool", comment);
   free(comment);
 
   /* Column for number of datasets used in this crop. */
   gal_list_data_add_alloc(&p->log, NULL, GAL_TYPE_UINT16, 1, &p->numout,
-                          NULL, 1, p->cp.minmapsize, "NUM_INPUTS", "count",
+                          NULL, 1, p->cp.minmapsize, p->cp.quietmmap,
+                          "NUM_INPUTS", "count",
                           "Number of input datasets used to make this crop.");
 
   /* Filename of crop. */
   gal_list_data_add_alloc(&p->log, NULL, GAL_TYPE_STRING, 1, &p->numout,
-                          NULL, 1, p->cp.minmapsize, "CROP_NAME", "name",
-                          "File name of crop.");
+                          NULL, 1, p->cp.minmapsize, p->cp.quietmmap,
+                          "CROP_NAME", "name", "File name of crop.");
 }
 
 
@@ -896,7 +897,8 @@ ui_preparations(struct cropparams *p)
           /* Set the basic information. */
           firsttype = p->type;
           firstndim = img->ndim;
-          p->bitnul = gal_fits_key_img_blank(p->type);
+          p->blankptrread  = gal_blank_alloc_write(p->type);
+          p->blankptrwrite = gal_fits_key_img_blank(p->type);
 
           /* Make sure the number of dimensions is supported. */
           if(firstndim>MAXDIM)
@@ -936,6 +938,13 @@ ui_preparations(struct cropparams *p)
     }
 
 
+  /* Polygon cropping is currently only supported on 2D */
+  if(p->imgs->ndim!=2 && p->polygon)
+    error(EXIT_FAILURE, 0, "%s: polygon cropping is currently only "
+          "supported on 2D datasets (images), not %zuD datasets",
+          p->imgs->name, p->imgs->ndim);
+
+
   /* Unify central crop methods into `p->centercoords'. */
   if(p->catname || p->center)
     ui_prepare_center(p);
@@ -1054,7 +1063,6 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct cropparams *p)
           gal_timing_report(NULL, msg, 1);
         }
     }
-
 }
 
 
@@ -1085,6 +1093,8 @@ ui_free_report(struct cropparams *p, struct timeval *t1)
   size_t i;
 
   /* Free the simple arrays (if they were set). */
+  free(p->blankptrread);
+  free(p->blankptrwrite);
   gal_data_free(p->center);
   if(p->cp.hdu) free(p->cp.hdu);
   if(p->cathdu) free(p->cathdu);
