@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2018, Free Software Foundation, Inc.
+Copyright (C) 2016-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -64,14 +64,14 @@ columns_alloc_radec(struct mkcatalogparams *p)
     for(i=0;i<p->objects->ndim;++i)
       gal_list_data_add_alloc(&p->wcs_vo, NULL, GAL_TYPE_FLOAT64, 1,
                               &p->numobjects, NULL, 0, p->cp.minmapsize,
-                              NULL, NULL, NULL);
+                              p->cp.quietmmap, NULL, NULL, NULL);
 
   /* For clumps */
   if(p->clumps && p->wcs_vc==NULL)
     for(i=0;i<p->objects->ndim;++i)
       gal_list_data_add_alloc(&p->wcs_vc, NULL, GAL_TYPE_FLOAT64, 1,
                               &p->numclumps, NULL, 0, p->cp.minmapsize,
-                              NULL, NULL, NULL);
+                              p->cp.quietmmap, NULL, NULL, NULL);
 }
 
 
@@ -89,14 +89,14 @@ columns_alloc_georadec(struct mkcatalogparams *p)
     for(i=0;i<p->objects->ndim;++i)
       gal_list_data_add_alloc(&p->wcs_go, NULL, GAL_TYPE_FLOAT64, 1,
                               &p->numobjects, NULL, 0, p->cp.minmapsize,
-                              NULL, NULL, NULL);
+                              p->cp.quietmmap, NULL, NULL, NULL);
 
   /* For clumps */
   if(p->clumps && p->wcs_gc==NULL)
     for(i=0;i<p->objects->ndim;++i)
       gal_list_data_add_alloc(&p->wcs_gc, NULL, GAL_TYPE_FLOAT64, 1,
                               &p->numclumps, NULL, 0, p->cp.minmapsize,
-                              NULL, NULL, NULL);
+                              p->cp.quietmmap, NULL, NULL, NULL);
 }
 
 
@@ -113,7 +113,7 @@ columns_alloc_clumpsradec(struct mkcatalogparams *p)
     for(i=0;i<p->objects->ndim;++i)
       gal_list_data_add_alloc(&p->wcs_vcc, NULL, GAL_TYPE_FLOAT64, 1,
                               &p->numobjects, NULL, 0, p->cp.minmapsize,
-                              NULL, NULL, NULL);
+                              p->cp.quietmmap, NULL, NULL, NULL);
 }
 
 
@@ -130,7 +130,7 @@ columns_alloc_clumpsgeoradec(struct mkcatalogparams *p)
     for(i=0;i<p->objects->ndim;++i)
       gal_list_data_add_alloc(&p->wcs_gcc, NULL, GAL_TYPE_FLOAT64, 1,
                               &p->numobjects, NULL, 0, p->cp.minmapsize,
-                              NULL, NULL, NULL);
+                              p->cp.quietmmap, NULL, NULL, NULL);
 }
 
 
@@ -1003,7 +1003,7 @@ columns_define_alloc(struct mkcatalogparams *p)
         case UI_KEY_STD:
           name           = "STD";
           unit           = MKCATALOG_NO_UNIT;
-          ocomment       = "Average of input standard deviation.";
+          ocomment       = "Sky standard deviation under object.";
           ccomment       = ocomment;
           otype          = GAL_TYPE_FLOAT32;
           ctype          = GAL_TYPE_FLOAT32;
@@ -1011,7 +1011,7 @@ columns_define_alloc(struct mkcatalogparams *p)
           disp_width     = 10;
           disp_precision = 4;
           oiflag[ OCOL_NUM    ] = ciflag[ CCOL_NUM    ] = 1;
-          oiflag[ OCOL_SUMSTD ] = ciflag[ CCOL_SUMSTD ] = 1;
+          oiflag[ OCOL_SUMVAR ] = ciflag[ CCOL_SUMVAR ] = 1;
           break;
 
         case UI_KEY_SEMIMAJOR:
@@ -1196,7 +1196,7 @@ columns_define_alloc(struct mkcatalogparams *p)
         {
           gal_list_data_add_alloc(&p->objectcols, NULL, otype, 1,
                                   &p->numobjects, NULL, 0, p->cp.minmapsize,
-                                  name, unit, ocomment);
+                                  p->cp.quietmmap, name, unit, ocomment);
           p->objectcols->status         = colcode->v;
           p->objectcols->disp_fmt       = disp_fmt;
           p->objectcols->disp_width     = disp_width;
@@ -1214,7 +1214,8 @@ columns_define_alloc(struct mkcatalogparams *p)
             {
               gal_list_data_add_alloc(&p->clumpcols, NULL, ctype, 1,
                                       &p->numclumps, NULL, 0,
-                                      p->cp.minmapsize, name, unit, ccomment);
+                                      p->cp.minmapsize, p->cp.quietmmap,
+                                      name, unit, ccomment);
               p->clumpcols->status         = colcode->v;
               p->clumpcols->disp_fmt       = disp_fmt;
               p->clumpcols->disp_width     = disp_width;
@@ -1738,7 +1739,8 @@ columns_fill(struct mkcatalog_passparams *pp)
           break;
 
         case UI_KEY_STD:
-          ((float *)colarr)[oind] = MKC_RATIO(oi[OCOL_SUMSTD], oi[OCOL_NUM]);
+          ((float *)colarr)[oind] = sqrt( MKC_RATIO( oi[ OCOL_SUMVAR ],
+                                                     oi[ OCOL_NUM    ]) );
           break;
 
         case UI_KEY_SEMIMAJOR:
@@ -1954,8 +1956,8 @@ columns_fill(struct mkcatalog_passparams *pp)
             break;
 
           case UI_KEY_STD:
-            ((float *)colarr)[cind] = MKC_RATIO( ci[ CCOL_SUMSTD ],
-                                                 ci[ CCOL_NUM ] );
+            ((float *)colarr)[cind] = sqrt( MKC_RATIO( ci[ CCOL_SUMVAR ],
+                                                       ci[ CCOL_NUM    ] ));
             break;
 
           case UI_KEY_SEMIMAJOR:
