@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2018, Free Software Foundation, Inc.
+Copyright (C) 2016-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -142,6 +142,7 @@ ui_initialize_options(struct mkcatalogparams *p,
         case GAL_OPTIONS_KEY_SEARCHIN:
         case GAL_OPTIONS_KEY_IGNORECASE:
         case GAL_OPTIONS_KEY_WORKOVERCH:
+        case GAL_OPTIONS_KEY_STDINTIMEOUT:
         case GAL_OPTIONS_KEY_INTERPNUMNGB:
         case GAL_OPTIONS_KEY_INTERPONLYBLANK:
           cp->coptions[i].flags=OPTION_HIDDEN;
@@ -383,7 +384,7 @@ ui_read_check_only_options(struct mkcatalogparams *p)
         {
           /* Allocate the data structure. */
           p->sky=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL, 0, -1,
-                                NULL, NULL, NULL);
+                                1, NULL, NULL, NULL);
 
           /* Write the value inside it. */
           *((float *)(p->sky->array))=tmp;
@@ -398,7 +399,7 @@ ui_read_check_only_options(struct mkcatalogparams *p)
         {
           /* Allocate the data structure. */
           p->std=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL, 0, -1,
-                                NULL, NULL, NULL);
+                                1, NULL, NULL, NULL);
 
           /* Write the value inside it. */
           *((float *)(p->std->array))=tmp;
@@ -531,17 +532,98 @@ ui_wcs_info(struct mkcatalogparams *p)
 
 
 
+static size_t
+ui_num_clumps(struct mkcatalogparams *p)
+{
+  char *basename;
+  int keepinputdir;
+  size_t i, counter, numclumps=0;
+  gal_list_i32_t *tmp, **labsinobj;
+  int32_t *o=p->objects->array, *of=o+p->objects->size, *c=p->clumps->array;
+
+  /* Allocate array of lists to keep the unique labels within each object. */
+  errno=0;
+  labsinobj=calloc(p->numobjects+1, sizeof *labsinobj);
+  if(labsinobj==NULL)
+    error(EXIT_FAILURE, 0, "%s: couldn't allocate %zu bytes for "
+          "`labsinobj'", __func__, p->numobjects * sizeof *labsinobj);
+
+  /* Go over each pixel and find the unique clump labels within each
+     object. */
+  do
+    {
+      /* Do the steps if we are on a clump. */
+      if(*o>0 && *c>0)
+        {
+          /* See if the label has already been found. */
+          for(tmp=labsinobj[*o];tmp!=NULL;tmp=tmp->next) if(tmp->v==*c) break;
+
+          /* When it wasn't found, `tmp==NULL'. */
+          if(tmp==NULL)
+            {
+              ++numclumps;
+              gal_list_i32_add(&labsinobj[*o], *c);
+            }
+        }
+
+      /* Increment the clumps pointer.*/
+      ++c;
+    }
+  while(++o<of);
+
+  /* Re-write the clump values so their numbering is contiguous, since this
+     is assumed during the later steps. */
+  o=p->objects->array;
+  c=p->clumps->array;
+  do
+    {
+      /* Do the steps if we are on a clump. */
+      if(*o>0 && *c>0)
+        {
+          counter=0;
+          for(tmp=labsinobj[*o];tmp!=NULL;tmp=tmp->next)
+            { counter++; if(tmp->v==*c) {*c=counter; break;} }
+        }
+
+      /* Increment the clumps pointer.*/
+      ++c;
+    }
+  while(++o<of);
+
+  /* Write the created file into a file for the user to inspect. */
+  keepinputdir=p->cp.keepinputdir;
+  p->cp.keepinputdir = p->cp.output ? 1 : 0;
+  basename = p->cp.output ? p->cp.output : p->objectsfile;
+  p->relabclumps=gal_checkset_automatic_output(&p->cp, basename,
+                                               "-clumps-relab.fits");
+  gal_fits_img_write(p->clumps, p->relabclumps, NULL, PROGRAM_STRING);
+  p->cp.keepinputdir=keepinputdir;
+
+  /* Clean up. */
+  for(i=0;i<p->numobjects;++i)
+    if(labsinobj[i]) gal_list_i32_free(labsinobj[i]);
+  free(labsinobj);
+
+  /* Return the number of clumps. */
+  return numclumps;
+}
+
+
+
+
+
 /* The only mandatory input is the objects image, so first read that and
    make sure its type is correct. */
 static void
 ui_read_labels(struct mkcatalogparams *p)
 {
-  char *msg;
   gal_data_t *tmp, *keys=gal_data_array_calloc(2);
 
   /* Read it into memory. */
-  p->objects = gal_array_read_one_ch(p->objectsfile, p->cp.hdu,
-                                     p->cp.minmapsize);
+  p->objects = gal_array_read_one_ch(p->objectsfile, p->cp.hdu, NULL,
+                                     p->cp.minmapsize, p->cp.quietmmap);
+  p->objects->ndim=gal_dimension_remove_extra(p->objects->ndim,
+                                              p->objects->dsize, NULL);
 
 
   /* Make sure it has an integer type. */
@@ -603,7 +685,10 @@ ui_read_labels(struct mkcatalogparams *p)
 
       /* Read the clumps image. */
       p->clumps = gal_array_read_one_ch(p->usedclumpsfile, p->clumpshdu,
-                                        p->cp.minmapsize);
+                                        NULL, p->cp.minmapsize,
+                                        p->cp.quietmmap);
+      p->clumps->ndim=gal_dimension_remove_extra(p->clumps->ndim,
+                                                 p->clumps->dsize, NULL);
 
       /* Check its size. */
       if( gal_dimension_is_different(p->objects, p->clumps) )
@@ -623,17 +708,7 @@ ui_read_labels(struct mkcatalogparams *p)
       keys[0].array=&p->clumpsn;            keys[1].array=&p->numclumps;
       gal_fits_key_read(p->usedclumpsfile, p->clumpshdu, keys, 0, 0);
       if(keys[0].status) p->clumpsn=NAN;
-      if(keys[1].status)
-        {
-          if( asprintf(&msg, "%s (hdu: %s): couldn't find/read `NUMLABS' in "
-                       "the header keywords, see CFITSIO error above. The "
-                       "clumps image must have the total number of clumps "
-                       "(irrespective of how many objects there are in the "
-                       "image) in this header keyword", p->usedclumpsfile,
-                       p->clumpshdu)<0 )
-            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
-          gal_fits_io_error(keys[1].status, msg);
-        }
+      if(keys[1].status) p->numclumps=ui_num_clumps(p);
 
       /* If there were no clumps, then free the clumps array and set it to
          NULL, so for the rest of the processing, MakeCatalog things that
@@ -708,7 +783,7 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
         case OCOL_VYY:                *values        = 1;          break;
         case OCOL_VXY:                *values        = 1;          break;
         case OCOL_SUMSKY:             *sky           = 1;          break;
-        case OCOL_SUMSTD:             *std           = 1;          break;
+        case OCOL_SUMVAR:             *std           = 1;          break;
         case OCOL_SUMWHT:             *values        = 1;          break;
         case OCOL_NUMWHT:             *values        = 1;          break;
         case OCOL_GX:                 /* Only object labels. */    break;
@@ -756,7 +831,7 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
           case CCOL_VYY:              *values        = 1;          break;
           case CCOL_VXY:              *values        = 1;          break;
           case CCOL_SUMSKY:           *sky           = 1;          break;
-          case CCOL_SUMSTD:           *std           = 1;          break;
+          case CCOL_SUMVAR:           *std           = 1;          break;
           case CCOL_SUMWHT:           *values        = 1;          break;
           case CCOL_NUMWHT:           *values        = 1;          break;
           case CCOL_GX:               /* Only clump labels. */     break;
@@ -871,6 +946,10 @@ ui_subtract_sky(struct mkcatalogparams *p)
     error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
           "the problem. For some reason, the size doesn't match", __func__,
           PACKAGE_BUGREPORT);
+
+  /* Inform the user that this operation is done (if necessary). */
+  if(!p->cp.quiet)
+    printf("  - Sky subtracted from input values.\n");
 }
 
 
@@ -904,8 +983,11 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
 
       /* Read the values dataset. */
       p->values=gal_array_read_one_ch_to_type(p->usedvaluesfile, p->valueshdu,
-                                              GAL_TYPE_FLOAT32,
-                                              p->cp.minmapsize);
+                                              NULL, GAL_TYPE_FLOAT32,
+                                              p->cp.minmapsize,
+                                              p->cp.quietmmap);
+      p->values->ndim=gal_dimension_remove_extra(p->values->ndim,
+                                                 p->values->dsize, NULL);
 
       /* Make sure it has the correct size. */
       if( gal_dimension_is_different(p->objects, p->values) )
@@ -951,8 +1033,11 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
 
           /* Read the Sky dataset. */
           p->sky=gal_array_read_one_ch_to_type(p->usedskyfile, p->skyhdu,
-                                               GAL_TYPE_FLOAT32,
-                                               p->cp.minmapsize);
+                                               NULL, GAL_TYPE_FLOAT32,
+                                               p->cp.minmapsize,
+                                               p->cp.quietmmap);
+          p->sky->ndim=gal_dimension_remove_extra(p->sky->ndim,
+                                                  p->sky->dsize, NULL);
 
           /* Check its size and prepare tile structure. */
           ui_preparation_check_size_read_tiles(p, p->sky, p->usedskyfile,
@@ -980,8 +1065,10 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
 
       /* Read the Sky standard deviation image into memory. */
       p->std=gal_array_read_one_ch_to_type(p->usedstdfile, p->stdhdu,
-                                           GAL_TYPE_FLOAT32,
-                                           p->cp.minmapsize);
+                                           NULL, GAL_TYPE_FLOAT32,
+                                           p->cp.minmapsize, p->cp.quietmmap);
+      p->std->ndim=gal_dimension_remove_extra(p->std->ndim,
+                                              p->std->dsize, NULL);
 
       /* Check its size and prepare tile structure. */
       ui_preparation_check_size_read_tiles(p, p->std, p->usedstdfile,
@@ -1014,7 +1101,11 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
 
           /* Read the mask image. */
           p->upmask = gal_array_read_one_ch(p->upmaskfile, p->upmaskhdu,
-                                            p->cp.minmapsize);
+                                            NULL, p->cp.minmapsize,
+                                            p->cp.quietmmap);
+          p->upmask->ndim=gal_dimension_remove_extra(p->upmask->ndim,
+                                                     p->upmask->dsize,
+                                                     NULL);
 
           /* Check its size. */
           if( gal_dimension_is_different(p->objects, p->upmask) )
@@ -1036,8 +1127,8 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
           /* Convert the mask to a uint8_t: with a 1 for all non-zero
              pixels and 0 for zero pixels. */
           zero=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, &one, NULL, 1, -1,
-                              NULL, NULL, NULL);
-          p->upmask=gal_arithmetic(GAL_ARITHMETIC_OP_NE,
+                              1, NULL, NULL, NULL);
+          p->upmask=gal_arithmetic(GAL_ARITHMETIC_OP_NE, 1,
                                    ( GAL_ARITHMETIC_INPLACE
                                      | GAL_ARITHMETIC_FREE
                                      | GAL_ARITHMETIC_NUMOK ),
@@ -1341,13 +1432,7 @@ ui_preparations_upperlimit(struct mkcatalogparams *p)
           "that is reported as the upper-limit");
 
   /* Set the random number generator. */
-  gsl_rng_env_setup();
-  p->rng=gsl_rng_alloc(gsl_rng_ranlxs1);
-  p->seed = ( p->envseed
-              ? gsl_rng_default_seed
-              : gal_timing_time_based_rng_seed() );
-  if(p->envseed) gsl_rng_set(p->rng, p->seed);
-  p->rngname=gsl_rng_name(p->rng);
+  p->rng=gal_checkset_gsl_rng(p->envseed, &p->rng_name, &p->rng_seed);
 
   /* Keep the minimum and maximum values of the random number generator. */
   p->rngmin=gsl_rng_min(p->rng);
@@ -1503,17 +1588,27 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct mkcatalogparams *p)
   ui_preparations(p);
 
 
+  /* If the output is a FITS table, prepare all the options as FITS
+     keywords to write in output later. */
+  if(gal_fits_name_is_fits(p->objectsout))
+      gal_options_as_fits_keywords(&p->cp);
+
+
   /* Inform the user. */
   if(!p->cp.quiet)
     {
       /* Write the information. */
-      printf(PROGRAM_NAME" started on %s", ctime(&p->rawtime));
+      printf(PROGRAM_NAME" "PACKAGE_VERSION" started on %s",
+             ctime(&p->rawtime));
       printf("  - Using %zu CPU thread%s\n", p->cp.numthreads,
              p->cp.numthreads==1 ? "." : "s.");
       printf("  - Objects: %s (hdu: %s)\n", p->objectsfile, p->cp.hdu);
       if(p->clumps)
         printf("  - Clumps:  %s (hdu: %s)\n", p->usedclumpsfile,
                p->clumpshdu);
+      if(p->relabclumps)
+        printf("  - RELABELED CLUMPS (no NUMLABS in original): %s\n",
+               p->relabclumps);
       if(p->values)
         printf("  - Values:  %s (hdu: %s)\n", p->usedvaluesfile,
                p->valueshdu);
@@ -1541,8 +1636,8 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct mkcatalogparams *p)
                p->upmaskfile, p->cp.hdu);
       if(p->upperlimit)
         {
-          printf("  - Random number generator name: %s\n", p->rngname);
-          printf("  - Random number generator seed: %"PRIu64"\n", p->seed);
+          printf("  - Random number generator name: %s\n", p->rng_name);
+          printf("  - Random number generator seed: %lu\n", p->rng_seed);
         }
     }
 }
