@@ -3,9 +3,9 @@ MakeCatalog - Make a catalog from an input and labeled image.
 MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -23,6 +23,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #ifndef UPPERLIMIT_H
 #define UPPERLIMIT_H
 
+void
+upperlimit_write_comments(struct mkcatalogparams *p,
+                          gal_list_str_t **comments, int withsigclip);
+
 void
 upperlimit_calculate(struct mkcatalog_passparams *pp);
 
