@@ -5,7 +5,7 @@ MakeNoise is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -284,8 +284,11 @@ ui_preparations(struct mknoiseparams *p)
 {
   /* Read the input image as a double type */
   p->input=gal_array_read_one_ch_to_type(p->inputname, p->cp.hdu, NULL,
-                                     GAL_TYPE_FLOAT64, p->cp.minmapsize);
+                                         GAL_TYPE_FLOAT64, p->cp.minmapsize,
+                                         p->cp.quietmmap);
   p->input->wcs=gal_wcs_read(p->inputname, p->cp.hdu, 0, 0, &p->input->nwcs);
+  p->input->ndim=gal_dimension_remove_extra(p->input->ndim, p->input->dsize,
+                                            p->input->wcs);
 
 
   /* If we are dealing with an input table, make sure the format of the
