@@ -60,7 +60,7 @@ statistics_pull_out_element(gal_data_t *input, size_t index)
 {
   size_t dsize=1;
   gal_data_t *out=gal_data_alloc(NULL, input->type, 1, &dsize,
-                                 NULL, 1, -1, NULL, NULL, NULL);
+                                 NULL, 1, -1, 1, NULL, NULL, NULL);
   memcpy( out->array,
           gal_pointer_increment(input->array, index, input->type),
           gal_type_sizeof(input->type) );
@@ -95,12 +95,13 @@ statistics_print_one_row(struct statisticsparams *p)
   double arg, *d;
   gal_list_i32_t *tmp;
   size_t dsize=1, counter;
-  gal_data_t *tmpv, *out=NULL, *num=NULL, *min=NULL, *max=NULL;
   gal_data_t *sum=NULL, *med=NULL, *meanstd=NULL, *modearr=NULL;
+  gal_data_t *tmpv, *sclip=NULL, *out=NULL, *num=NULL, *min=NULL, *max=NULL;
 
   /* The user can ask for any of the operators more than once, also some
      operators might return more than one usable value (like mode). So we
-     will calculate the desired values once, and then print them. */
+     will calculate the desired values once, and then print them any number
+     of times. */
   for(tmp=p->singlevalue; tmp!=NULL; tmp=tmp->next)
     switch(tmp->v)
       {
@@ -125,11 +126,21 @@ statistics_print_one_row(struct statisticsparams *p)
       case UI_KEY_MODEQUANT:
       case UI_KEY_MODESYM:
       case UI_KEY_MODESYMVALUE:
-        modearr = ( modearr ? modearr
+        modearr = ( modearr
+                    ? modearr
                     : gal_statistics_mode(p->sorted, p->mirrordist, 0) );
         d=modearr->array;
         if(d[2]<GAL_STATISTICS_MODE_GOOD_SYM) d[0]=d[1]=NAN;
         break;
+      case UI_KEY_SIGCLIPSTD:
+      case UI_KEY_SIGCLIPMEAN:
+      case UI_KEY_SIGCLIPNUMBER:
+      case UI_KEY_SIGCLIPMEDIAN:
+        sclip = ( sclip
+                  ? sclip
+                  : gal_statistics_sigma_clip(p->sorted, p->sclipparams[0],
+                                              p->sclipparams[1], 0, 1) );
+        break;
 
       /* Will be calculated as printed. */
       case UI_KEY_QUANTILE:
@@ -172,6 +183,14 @@ statistics_print_one_row(struct statisticsparams *p)
           out=statistics_pull_out_element(modearr, 2); mustfree=1; break;
         case UI_KEY_MODESYMVALUE:
           out=statistics_pull_out_element(modearr, 3); mustfree=1; break;
+        case UI_KEY_SIGCLIPSTD:
+          out=statistics_pull_out_element(sclip,   3); mustfree=1; break;
+        case UI_KEY_SIGCLIPMEAN:
+          out=statistics_pull_out_element(sclip,   2); mustfree=1; break;
+        case UI_KEY_SIGCLIPMEDIAN:
+          out=statistics_pull_out_element(sclip,   1); mustfree=1; break;
+        case UI_KEY_SIGCLIPNUMBER:
+          out=statistics_pull_out_element(sclip,   0); mustfree=1; break;
 
         /* Not previously calculated. */
         case UI_KEY_QUANTILE:
@@ -182,7 +201,7 @@ statistics_print_one_row(struct statisticsparams *p)
         case UI_KEY_QUANTFUNC:
           arg = statistics_read_check_args(p);
           tmpv = gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &dsize,
-                                NULL, 1, -1, NULL, NULL, NULL);
+                                NULL, 1, -1, 1, NULL, NULL, NULL);
           *((double *)(tmpv->array)) = arg;
           tmpv = gal_data_copy_to_new_type_free(tmpv, p->input->type);
           out = gal_statistics_quantile_function(p->sorted, tmpv, 0);
@@ -213,6 +232,7 @@ statistics_print_one_row(struct statisticsparams *p)
   if(max)     gal_data_free(max);
   if(sum)     gal_data_free(sum);
   if(med)     gal_data_free(med);
+  if(sclip)   gal_data_free(sclip);
   if(meanstd) gal_data_free(meanstd);
   if(modearr) gal_data_free(modearr);
 }
@@ -319,7 +339,8 @@ statistics_on_tile(struct statisticsparams *p)
 
       /* Allocate the space necessary to keep the value for each tile. */
       values=gal_data_alloc(NULL, type, p->input->ndim, tl->numtiles, NULL,
-                            0, p->input->minmapsize, NULL, NULL, NULL);
+                            0, p->input->minmapsize, p->cp.quietmmap,
+                            NULL, NULL, NULL);
 
       /* Read the argument for those operations that need it. This is done
          here, because below, the functions are repeated on each tile. */
@@ -331,7 +352,7 @@ statistics_on_tile(struct statisticsparams *p)
         case UI_KEY_QUANTFUNC:
           arg = statistics_read_check_args(p);
           tmpv = gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &dsize,
-                                NULL, 1, -1, NULL, NULL, NULL);
+                                NULL, 1, -1, 1, NULL, NULL, NULL);
           *((double *)(tmpv->array)) = arg;
           tmpv = gal_data_copy_to_new_type_free(tmpv, p->input->type);
         }
@@ -491,7 +512,7 @@ set_bin_range_params(struct statisticsparams *p)
   if(p->manualbinrange)
     {
       /* Allocate the range data structure. */
-      range=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &rsize, NULL, 0, -1,
+      range=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &rsize, NULL, 0, -1, 1,
                            NULL, NULL, NULL);
       ((float *)(range->array))[0]=p->greaterequal;
       ((float *)(range->array))[1]=p->lessthan;
@@ -695,7 +716,7 @@ print_mirror_hist_cfp(struct statisticsparams *p)
   gal_data_t *table;
   double mirror_val;
   gal_data_t *mirror=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &dsize,
-                                    NULL, 1, -1, NULL, NULL, NULL);
+                                    NULL, 1, -1, 1, NULL, NULL, NULL);
 
   /* Convert the given mirror value into the type of the input dataset. */
   *((double *)(mirror->array)) = p->mirror;
