@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -29,6 +29,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <unistd.h>
 #include <sys/stat.h>
+#include <sys/fcntl.h>
 
 #include <gnuastro/data.h>
 
@@ -291,7 +292,7 @@ gal_checkset_not_dir_part(char *filename)
 
 
 
-/* Check if a file exists and report if it doesn't: */
+/* Check if a file exists and report if it doesn't. */
 void
 gal_checkset_check_file(char *filename)
 {
@@ -333,6 +334,59 @@ gal_checkset_check_file_return(char *filename)
 
 
 
+/* If a file doesn't exist and its directory is writable, return
+   1. Otherwise, return 0. */
+int
+gal_checkset_writable_notexist(char *filename)
+{
+  int out=1;
+  char *dir;
+  FILE *tmpfile;
+
+  /* If the filename is `NULL' everything is ok (it doesn't exist)! In some
+     cases, a NULL filename is interpretted to mean standard output. */
+  if(filename==NULL) return 1;
+
+  /* We want to make sure that we can open and write to this file. But
+     the user might have asked to not delete the file, so the
+     contents should not be changed. Therefore we have to open it with
+     `r+`. */
+  errno=0;
+  tmpfile=fopen(filename, "r+");
+  if (tmpfile)                        /* The file opened. */
+    {
+      /* Close the file. */
+      errno=0;
+      if(fclose(tmpfile))
+        error(EXIT_FAILURE, errno, "%s", filename);
+
+      /* The file exists, return 0. */
+      out=0;
+    }
+
+  /* If the file doesn't exist, we just need to make sure if we have write
+     permissions to its host directory. */
+  else
+    {
+      /* Separate the directory part of the filename. */
+      dir=gal_checkset_dir_part(filename);
+
+      /* See if this directory is writable by this user. */
+      errno=0;
+      if( access(dir, W_OK) ) out=0;
+
+      /* Clean up. */
+      free(dir);
+    }
+
+  /* Return the final value. */
+  return out;
+}
+
+
+
+
+
 /* Check if a file exists and can be opened. If the `keep' value is
    non-zero, then the file will remain untouched, otherwise, it will be
    deleted (since most programs need to make a clean output). When the file
@@ -591,16 +645,26 @@ gal_checkset_check_dir_write_add_slash(char **dirname)
 
 
 
-/* If the given directory exists, then nothing is done, if it doesn't, it
-   will be created. */
-void
+/* If the given directory exists and is writable, then nothing is done. If
+   it doesn't, it will be created. If it fails at creating the file, or the
+   file isn't writable it returns a non-zero value: the errno, which can be
+   directly used in `error'. */
+int
 gal_checkset_mkdir(char *dirname)
 {
+  int errnum=0;
   struct stat st={0};
   if( stat(dirname, &st) == -1 )
     {
       errno=0;
       if( mkdir(dirname, 0700) == -1 )
-        error(EXIT_FAILURE, errno, "making %s", dirname);
+        errnum=errno;
     }
+  else
+    {
+      errno=0;
+      if( faccessat(AT_FDCWD, dirname, W_OK, AT_EACCESS) == -1 )
+        errnum=errno;
+    }
+  return errnum;
 }
