@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -485,7 +485,7 @@ gal_wcs_on_tile(gal_data_t *tile)
 double *
 gal_wcs_warp_matrix(struct wcsprm *wcs)
 {
-  double *out;
+  double *out, crota2;
   size_t i, j, size=wcs->naxis*wcs->naxis;
 
   /* Allocate the necessary array. */
@@ -507,6 +507,33 @@ gal_wcs_warp_matrix(struct wcsprm *wcs)
       for(i=0;i<size;++i)
         out[i]=wcs->cd[i];
     }
+  else if(wcs->altlin & 0x4)     /* Has CROTAi array.   */
+    {
+      /* Basic sanity checks. */
+      if(wcs->naxis!=2)
+        error(EXIT_FAILURE, 0, "%s: CROTAi currently on works in 2 "
+              "dimensions.", __func__);
+      if(wcs->crota[0]!=0.0)
+        error(EXIT_FAILURE, 0, "%s: CROTA1 is not zero", __func__);
+
+      /* CROTAi keywords are depreciated in the FITS standard. However, old
+         files may still use them. For a full description of CROTAi
+         keywords and their history (along with the conversion equations
+         here), please see the link below:
+
+         https://fits.gsfc.nasa.gov/users_guide/users_guide/node57.html
+
+         Just note that the equations of the link above convert CROTAi to
+         PC. But here we want the "final" matrix (after multiplication by
+         the `CDELT' values). So to speed things up, we won't bother
+         dividing and then multiplying by the same CDELT values in the
+         off-diagonal elements. */
+      crota2=wcs->crota[1];
+      out[0] = wcs->cdelt[0] * cos(crota2);
+      out[1] = -1 * wcs->cdelt[1] *sin(crota2);
+      out[2] = wcs->cdelt[0] * sin(crota2);
+      out[3] = wcs->cdelt[1] * cos(crota2);
+    }
   else
     error(EXIT_FAILURE, 0, "%s: currently only PCi_ja and CDi_ja keywords "
           "are recognized", __func__);
@@ -537,11 +564,15 @@ gal_wcs_decompose_pc_cdelt(struct wcsprm *wcs)
   double *ps;
   size_t i, j;
 
+  /* If there is on WCS, then don't do anything. */
+  if(wcs==NULL) return;
+
   /* The correction is only needed when the PC matrix is filled. */
   if(wcs->pc)
     {
       /* Get the pixel scale. */
       ps=gal_wcs_pixel_scale(wcs);
+      if(ps==NULL) return;
 
       /* The PC matrix and the CDELT elements might both contain scale
          elements (during processing the scalings might be added only to PC
@@ -609,6 +640,7 @@ gal_wcs_angular_distance_deg(double r1, double d1, double r2, double d2)
 
 
 
+
 /* Return the pixel scale of the dataset in units of the WCS. */
 double *
 gal_wcs_pixel_scale(struct wcsprm *wcs)
@@ -616,20 +648,30 @@ gal_wcs_pixel_scale(struct wcsprm *wcs)
   gsl_vector S;
   gsl_matrix A, V;
   int warning_printed;
-  size_t i, j, maxj, n=wcs->naxis;
-  double jvmax, *a, *out, maxrow, minrow;
-  double *v=gal_pointer_allocate(GAL_TYPE_FLOAT64, n*n, 0, __func__, "v");
-  size_t *permutation=gal_pointer_allocate(GAL_TYPE_SIZE_T, n, 0, __func__,
-                                           "permutation");
-  gal_data_t *pixscale=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &n, NULL,
-                                      0, -1, NULL, NULL, NULL);
+  gal_data_t *pixscale;
+  size_t i, j, n, maxj, *permutation;
+  double jvmax, *a, *out, *v, maxrow, minrow;
 
+  /* Only continue if a WCS exists. */
+  if(wcs==NULL) return NULL;
 
   /* Write the full WCS rotation matrix into an array, irrespective of what
      style it was stored in the wcsprm structure (`PCi_j' style or `CDi_j'
      style). */
   a=gal_wcs_warp_matrix(wcs);
 
+  /* A small sanity check (this won't work on a singular matrix, can happen
+     in FITS WCSs!). In this case, we should return NULL.*/
+  n=wcs->naxis;
+  for(i=0;i<n;++i) {if(a[i*n+i]==0.0f) return NULL;}
+
+  /* Now that everything is good, we can allocate the necessary memory. */
+  v=gal_pointer_allocate(GAL_TYPE_FLOAT64, n*n, 0, __func__, "v");
+  permutation=gal_pointer_allocate(GAL_TYPE_SIZE_T, n, 0, __func__,
+                                   "permutation");
+  pixscale=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &n, NULL,
+                          0, -1, 1, NULL, NULL, NULL);
+
 
   /* To avoid confusing issues with floating point errors being written in
      the non-diagonal elements of the FITS header PC or CD matrices, we
@@ -765,6 +807,7 @@ gal_wcs_pixel_area_arcsec2(struct wcsprm *wcs)
 
   /* Get the pixel scales along each axis in degrees, then multiply. */
   pixscale=gal_wcs_pixel_scale(wcs);
+  if(pixscale==NULL) return NAN;
 
   /* Clean up and return the result. */
   out = pixscale[0] * pixscale[1] * 3600.0f * 3600.0f;
@@ -900,7 +943,8 @@ wcs_convert_prepare_out(gal_data_t *coords, struct wcsprm *wcs, int inplace)
     for(i=0;i<wcs->naxis;++i)
       gal_list_data_add_alloc(&out, NULL, GAL_TYPE_FLOAT64, 1,
                               &coords->size, NULL, 0, coords->minmapsize,
-                              wcs->ctype[i], wcs->cunit[i], NULL);
+                              coords->quietmmap, wcs->ctype[i], wcs->cunit[i],
+                              NULL);
   return out;
 }
 
